# -------------------------------------------------------------------------------------
# MKDoc
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      This module takes care of variables initialization, and then triggers each
#      plugin module until one of them handles the query
#
# -------------------------------------------------------------------------------------
package MKDoc;
use lib qw /../;
use MKDoc::Functions;
use flo::Template;
use flo::Standard;
use lib::sql::DBH;
use lib::sql::Table;
use CGI;
use CGI::Carp;
use Data::Dumper;
use strict;



=cut

We directly use the stash for global variables...

Conventions:
$::{P_MKD_SomeVariableName} # persistent global variable
$::{S_MKD_SomeVariableName} # server-specific persistent global variable
$::{MKD_SomeVariableName}   # global variable reinitialized on each request

$::{P_MKD_DIR}    # MKDoc directory, i.e. /opt/mkdoc
$::{P_MKD_IMP}    # Modules which have been dynamically imported
$::{P_MKD_NFO}    # Site information: hostname => directory mapping
$::{MKD_CFG}      # cgi.mkdoc or cgi.admin config object

=cut


BEGIN
{
    # find out where the MKDoc code base directory is, and stores that
    # information in $::{P_MKD_DIR}
    my $mkdoc_base_dir = undef;
    if ($ENV{SCRIPT_FILENAME})
    {
        my @tmp = split /\//, $ENV{SCRIPT_FILENAME};
        pop @tmp;
        pop @tmp;
	$mkdoc_base_dir = join '/', @tmp;
    }
    else
    {
        $mkdoc_base_dir = '..';
    }
    
    # initialize variables common to all sites which do not need to
    # be reinitialized across connections
    $::{P_MKD_DIR} = $mkdoc_base_dir;
    $::{P_MKD_IMP} = {};
    
    # load sites information from $::{P_MKD_DIR}/conf/sites.conf
    my $VAR1 = undef;
    open FP, "<$::{P_MKD_DIR}/conf/sites.conf" or return;
    my $data = join '', <FP>;
    close FP;
    eval $data;
    confess ($@) if (defined $@ and $@);
    $::{P_MKD_NFO} = $VAR1;
}


##
# $class->process ($conf_file);
# -----------------------------
#   Processes this query with the modules defined in $conf_file
##
sub process
{
    my $class = shift;
    my $conf_file = shift;
    
    $::{MKD_CFG} = new lib::Config ($conf_file);
    clean_query_globals($conf_file);
    init ($conf_file);
    main ($conf_file);
    clean_query_globals($conf_file);
}


##
# clean_query_globals();
# ----------------------
#   This subroutines reinitialises all the global scalar variables
#   which we DO NOT want to be persistent across multiple queries
#
#   By convention they start with 'MKD_' and are stored in $::
##
sub clean_query_globals
{
    my $conf_file = shift;
    foreach my $key (keys %::)
    {
	if ($key =~ /^MKD_/)
	{
	    if (ref $::{$key} eq 'HASH')   { $::{$key}    = {};    next }
	    if (ref $::{$key} eq 'ARRAY')  { $::{$key}    = [];    next }
	    next if (ref $::{$key} eq 'lib::Config'); # handled by process()
	    $::{$key} = undef;
	}
    }
}


##
# init();
# -------
#   This function makes sure that classes are properly initialized
#   before we start processing. This is to ensure that class attributes
#   are properly initialized when running under speedycgi or mod_perl
##
sub init
{
    my $conf_file = shift;
    $CGI::USE_PARAM_SEMICOLONS = 1;
    
    # if for some reason the database handler was not disconnected
    # from the previous query, let's do it now
    + lib::sql::DBH->disconnect;
    
    # reinitializes global variables to their default values
    $lib::PSP::BASE_DIR              = undef;
    $lib::Template::CONFIG           = undef;
    $lib::Template::LEFT_TOKEN       = undef;
    $lib::Template::RIGHT_TOKEN      = undef;
    $lib::Template::BASE_DIR         = undef;
    $lib::Template::TOKEN_REGEX      = undef;
    $lib::Template::STASH            = {};
    
    # initializes template directory and delimiters
    init flo::Template;
    
    # loads database table schemas
    + lib::sql::Table->load_state (local_base_dir() .'/su');
    
    # sets up the document table to the current document language,
    # provided that we're on some kind of document
    my $document_table = table ('Document');
    $document_table->lang (undef);
    my $current_document = current_document();
    (defined $current_document) and $document_table->lang ($current_document->Lang);
}


##
# Processes the query. Triggers the main() class method of the
# plugins in the order they were defined in the config file. Plugins
# can terminate the query processing by sending the 'TERMINATE' string
# back
##
sub main
{
    foreach my $pkg ($::{MKD_CFG}->keys)
    {
        unless (defined $::{P_MKD_IMP}->{$pkg})
        {
	    my $file = $pkg;
            $file =~ s/::/\//g;
            $file .= '.pm';
	    require $file; 
	    import $pkg;
	    $::{P_MKD_IMP}->{$pkg} = 1;
        }
	
	my $ret = $pkg->main;
	last if (defined $ret and $ret eq 'TERMINATE');
    }
    
    + lib::sql::DBH->disconnect;
    my $document_table = table ('Document');
    $document_table->lang (undef);
}


1;




