# -------------------------------------------------------------------------------------
# MKDoc::XML::Dumper
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      This module turns perl structures and objects into XML files and reversely.
#
# -------------------------------------------------------------------------------------
package MKDoc::XML::Dumper;
use XML::Parser;
use Data::Dumper;
use Exporter;
use overload;
use strict;
use 5.6.0;
use utf8;
use UTF8::Hack;
use Unicode::String qw /latin1/;
use Carp;

our @ISA = qw /Exporter/;
our @EXPORT = qw /xml2pl pl2xml/;


##
# $class->new;
# ------------
#   Returns a new MKDoc::XML::Dumper object
##
sub new
{
    my $class = shift;
    $class = ref $class || $class;
    my $self = {};
    return bless $self, $class;
}


##
# $self->_pl2xml ($ref);
# ----------------------
#   Returns $ref as XML data
##
sub _pl2xml
{
    my ($obj,$ref) = @_;
    return $obj->_pl2xml_string($ref);
}


##
# pl2xml ($ref);
# --------------
#   Returns $ref dumped as an XML structure
##
sub pl2xml
{
    my $obj = shift;
    my $dmp = new 'MKDoc::XML::Dumper';
    return $dmp->_pl2xml ($obj);
}


##
# xml2pl ($xml);
# --------------
#   Parses $xml and returns the corresponding Perl structure / objects
##
sub xml2pl
{
    my $data = shift;
    my $Parser = XML::Parser->new (Style => 'Tree');
    my $Tree;
    eval {
	$Tree = $Parser->parse ($data);
    };
    confess ($@) if (defined $@ and $@);
    
    my $Dumper = new 'MKDoc::XML::Dumper';
    return $Dumper->_xml2pl($Tree);
}


##
# $self->_pl2xml_string ($ref);
# -----------------------------
#   Returns $ref as XML data
##
sub _pl2xml_string
{
    my ($obj, $ref) = @_;
    return(
	   '<?xml version="1.0" encoding="UTF-8"?>' . "\n" .
	   "<perl>" .
	   &Tree2XML($ref, 1) .
	   "\n</perl>\n"
	   );
}


##
# Tree2XML
# --------
#   Returns $ref as XML data
##
sub Tree2XML
{
    my ($ref, $indent, $backref) = @_;
    my $string = '';
    $backref ||= {};
    
    if (ref $ref)
    {
	# SCALAR REFERENCE
	if (Tree2XML_is_scalar_ref ($ref))
	{
	    $string .= "\n" . Tree2XML_scalar_ref_dump ($ref, $indent, $backref);
	}
	
	# HASH REFERENCE
	elsif (Tree2XML_is_hash_ref ($ref))
	{
	    $string .= "\n" . Tree2XML_hash_ref_dump ($ref, $indent, $backref);
	}
	
	# ARRAY REFERENCE 
	elsif (Tree2XML_is_array_ref())
	{
	    $string .= "\n" . Tree2XML_array_ref_dump ($ref, $indent, $backref);
	}
    }
    else
    {
	if (defined $ref) { $string .= "\n" . "  " x $indent . "<scalar>" . &_to_UTF8_XML($ref) . "</scalar>" }
	else              { $string .= "\n" . "  " x $indent . "<scalar undef=\"1\"/>" }
    }
    
    return ($string);
}


##
# Tree2XML_scalar_ref_dump ($ref, $indent);
# -----------------------------------------
#   Dumps a scalar ref
##
sub Tree2XML_scalar_ref_dump
{
    my $ref = shift;
    my $indent = shift;
    my $backref = shift;
    
    my $id = 'id_' . ( 0 + $ref );
    
   return "  " x $indent . "<scalarref backref=\"$id\"/>"
        if (defined $backref->{$id} and $backref->{$id});
    $backref->{$id} = 1;
    
    my $ref_ref = ref $ref;
    
  CASE:
    (($ref_ref eq 'SCALAR') and (defined $$ref)) and do
    {
        return "  " x $indent . "<scalarref id=\"$id\">" . &_to_UTF8_XML ($$ref) . "</scalarref>";	
    };
    
    (($ref_ref eq 'SCALAR') and (not defined $$ref)) and do
    {
        return "  " x $indent . "<scalarref id=\"$id\" undef=\"1\"/>";
    };
    
    (($ref_ref ne 'SCALAR') and (defined $$ref)) and do
    {
        return "  " x $indent . "<scalarref id=\"$id\" bless=\"$ref_ref\">" . &_to_UTF8_XML($$ref) . "</scalarref>";
    };
    
    (($ref_ref ne 'SCALAR') and (not defined $$ref)) and do
    {
        return "  " x $indent . "<scalarref id=\"$id\" undef=\"1\" bless=\"$ref_ref\"/>";
    };
}


##
# Tree2XML_hash_ref_dump ($ref, $indent);
# ---------------------------------------
#   Dumps a hash ref
##
sub Tree2XML_hash_ref_dump
{
    my $ref = shift;
    my $indent = shift;
    my $backref = shift;
    my $string = '';
    
    my $id = 'id_' . ( 0 + $ref );
    
    # if this reference has already been dumped,
    # then let's return it
    return "  " x $indent . "<hash backref=\"$id\"/>"
        if (defined $backref->{$id} and $backref->{$id});
    $backref->{$id} = 1;
    
    # if this reference is an object, we must add a bless="..."
    # attribute to bless it back to its package when the file
    # is undumped
    if (ref $ref eq 'HASH')
    {
	$string .= "  " x $indent . "<hash id=\"$id\">";
    }
    else
    {
        my $ref_ref = ref $ref;
        $string .= "  " x $indent . "<hash id=\"$id\" bless=\"$ref_ref\">";
    }
    
    $indent++;
    foreach my $key (sort keys %{$ref})
    {
	unless (defined $ref->{$key})
	{
	    $string .= "\n" . "  " x $indent . "<item key=\"" . &_to_UTF8_XML($key) . "\" undef=\"1\"/>";
	    next;
	}
	
        $string .= "\n" . "  " x $indent . "<item key=\"" . &_to_UTF8_XML($key) . "\">";
        if (ref($ref->{$key}))
        {
            $string .= &Tree2XML ( $ref->{$key}, $indent+1, $backref );
            $string .= "\n" . "  " x $indent . "</item>";
        }
        else
        {
            $string .= &_to_UTF8_XML ($ref->{$key}) . "</item>";
        }
    }
    $indent--;
    $string .= "\n" . "  " x $indent . "</hash>";
}


##
# Tree2XML_array_ref_dump ($ref, $indent);
# ---------------------------------------
#   Dumps an array ref
##
sub Tree2XML_array_ref_dump
{
    my $ref = shift;
    my $indent = shift;
    my $backref = shift;
    
    my $string = '';
    
    my $id = 'id_' . ( 0 + $ref );
    
    return "  " x $indent . "<array backref=\"$id\"/>"
        if (defined $backref->{$id} and $backref->{$id});
    $backref->{$id} = 1;
   
    if (ref $ref eq 'ARRAY')
    {
        $string .= "  " x $indent . "<array id=\"$id\">";
    }
    else
    {
        my $ref_ref = ref $ref;
        $string .= "  " x $indent . "<array id=\"$id\" bless=\"$ref_ref\">";
    }
    
    $indent++;
    for (my $i=0; $i < @$ref; $i++)
    {
	unless (defined $ref->[$i])
	{
	    $string .= "\n" . "  " x $indent . "<item key=\"$i\" undef=\"1\"/>";
	    next;
	}
	
        $string .= "\n" . "  " x $indent . "<item key=\"$i\">";
        if (ref($ref->[$i]))
        {
            $string .= &Tree2XML ( $ref->[$i], $indent+1, $backref );
            $string .= "\n" . "  " x $indent . "</item>";
        }
        else
        {
            $string .= &_to_UTF8_XML ($ref->[$i]) . "</item>";
        }
    }
    $indent--;
    $string .= "\n" . "  " x $indent . "</array>";
}


##
# Tree2XML_is_scalar_ref;
# -----------------------
#   Returns TRUE is $ref is a scalar ref,
#   FALSE otherwise
##
sub Tree2XML_is_scalar_ref
{
    my $ref = shift;
    return unless (defined ref $ref);
    return unless (ref $ref);
    return $ref =~ /SCALAR\(/;
}


##
# Tree2XML_is_hash_ref;
# -----------------------
#   Returns TRUE is $ref is a hash ref,
#   FALSE otherwise
##
sub Tree2XML_is_hash_ref
{
    my $ref = shift;
    return unless (defined ref $ref);
    return unless (ref $ref);
    return $ref =~ /HASH\(/;
}


##
# Tree2XML_is_array_ref;
# -----------------------
#   Returns TRUE is $ref is an array ref,
#   FALSE otherwise
##
sub Tree2XML_is_array_ref
{
    my $ref = shift;
    return unless (defined ref $ref);
    return unless (ref $ref);
    return $ref =~ /ARRAY\(/;
}


##
# to_utf8_xml ($data);
# --------------------
#   Turns $data into UTF8 which is XML encoded
##
sub _to_UTF8_XML
{
    my $data = shift;
    unless (UTF8::Hack::isutf8 ($data))
    {
        my $u_str = latin1 ($data);
        $data = $u_str->utf8;
        + UTF8::Hack::utf8on ($data);
    }
    
    $data =~ s/&/&amp;/g;
    $data =~ s/</&lt;/g;
    $data =~ s/>/&gt;/g;
    $data =~ s/\'/&apos;/g;
    $data =~ s/\"/&quot;/g;
    return $data;
}


sub _xml2pl
{
    my ($obj,$tree) = @_;
    
    ## Skip enclosing "perl" level
    my $TopItem = $tree->[1];
    my $ref = &Undump($TopItem);
    
    return($ref);
}


## Undump
## Takes a parse tree of the XML generated by _pl2xml, and recursively
## undumps it to create a data structure in memory.  The top-level
## object is a scalar, a reference to a scalar, a hash, or an array.
## Hashes and arrays may themselves contain scalars, or references to
## scalars, or references to hashes or arrays.
sub Undump
{
    my ($Tree) = shift;
    my $backref = shift || {};
    my $ref = undef;
    my $FoundScalar;
    my $i;
    
    my $foundUndef = 0;
    
    for ($i = 1; $i < $#$Tree; $i+=2)
    {
	# if we have a backref, let's take it from the
	# $backref hashref
	if (lc($Tree->[$i]) eq "scalar")
	{
	    # if the value was undef, let's do something
	    # set $ref to undef rather than a blank string
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"undef"} )
	    {
		$foundUndef = 1;
		$ref = undef;
	    }
	    else
	    {
		## Make a copy of the string
		$ref = $Tree->[$i+1]->[2];
	    }
	    last;
	}
	
	
	if (lc($Tree->[$i]) eq "scalarref")
	{
	    # if this is a backreference, let's use it
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"backref"} )
	    {
		my $id = $Tree->[$i+1]->[0]->{"backref"};
		$ref = $backref->{$id};
		last;
	    }
	    
	    # if the value was undef, let's do something
	    # set $ref to undef rather than a blank string
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"undef"} )
	    {
		$foundUndef = 1;
		my $undef = undef;
		$ref = \$undef;
	    }
	    else
	    {
		## Make a ref to a copy of the string
		$ref = \ "$Tree->[$i+1]->[2]";
	    }
	    
	    # if there is an ID for this reference, then
	    # we should save it in $backref
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"id"} )
	    {
		my $id = $Tree->[$i+1]->[0]->{"id"};
		$backref->{$id} = $ref;
	    }
	    
	    # if this scalar ref was an object,
	    # let's bless it to its package
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"bless"} )
	    {
		bless $ref, $Tree->[$i+1]->[0]->{"bless"};
	    }
	    last;
	};
	
	
	if (lc($Tree->[$i]) eq "hash")
	{
	    $ref = {};
	    
	    # if this is a backreference, let's use it
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"backref"} )
	    {
		my $id = $Tree->[$i+1]->[0]->{"backref"};
		$ref = $backref->{$id};
		last;
	    }
	    
	    # if there is an ID for this reference, then
	    # we should save it in $backref
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"id"} )
	    {
		my $id = $Tree->[$i+1]->[0]->{"id"};
		$backref->{$id} = $ref;
	    }
	    
	    my $j;
	    for ($j = 1; $j < $#{$Tree->[$i+1]}; $j+=2)
	    {
		next unless $Tree->[$i+1]->[$j] eq "item";
		my $ItemTree = $Tree->[$i+1]->[$j+1];
		next unless defined(my $key = $ItemTree->[0]->{key});
		
		if (defined $ItemTree->[0]->{'undef'}) { $ref->{$key} = undef }
		else { $ref->{$key} = &Undump ($ItemTree, $backref) }
	    }
	    
	    # if this scalar ref was an object,
	    # let's bless it to its package
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"bless"} )
	    {
		bless $ref, $Tree->[$i+1]->[0]->{"bless"};
	    }
	    
	    last;
	}
	
	
	if (lc($Tree->[$i]) eq "array")
	{
	    $ref = [];
	    
	    # if this is a backreference, let's use it
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"backref"} )
	    {
		my $id = $Tree->[$i+1]->[0]->{"backref"};
		$ref = $backref->{$id};
		last;
	    }

	    # if there is an ID for this reference, then
	    # we should save it in $backref
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"id"} )
	    {
		my $id = $Tree->[$i+1]->[0]->{"id"};
		$backref->{$id} = $ref;
	    }
	    
	    my $j;
	    for ($j = 1; $j < $#{$Tree->[$i+1]}; $j+=2)
	    {
		next unless $Tree->[$i+1]->[$j] eq "item";
		my $ItemTree = $Tree->[$i+1]->[$j+1];
		next unless defined(my $key = $ItemTree->[0]->{key});
		
		if (defined $ItemTree->[0]->{'undef'}) { $ref->[$key] = undef }
		else { $ref->[$key] = &Undump ($ItemTree, $backref) }
	    }
	    
	    # if this scalar ref was an object,
	    # let's bless it to its package
	    if ( defined $Tree->[$i+1]      and
		 defined $Tree->[$i+1]->[0] and
		 defined $Tree->[$i+1]->[0]->{"bless"} )
	    {
		bless $ref, $Tree->[$i+1]->[0]->{"bless"};
	    }
	    
	    last;
	}
	
	if (lc($Tree->[$i]) eq "0")
	{
	    $FoundScalar = $Tree->[$i + 1] unless defined $FoundScalar;
	};
    }
    
    ## If $ref is not set at this point, it means we've just
    ## encountered a scalar value directly inside the item tag.
    return $ref if ($foundUndef);
    $ref = $FoundScalar unless defined($ref);
    
  done:
    return ($ref);
}


1;

