# -------------------------------------------------------------------------------------
# MKDoc::XML::XHTML::Linkifier
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      A class to linkify XHTML expressions
#
# -------------------------------------------------------------------------------------
package MKDoc::XML::XHTML::Linkifier;
use MKDoc::XML::Tree;
use MKDoc::Unicode;
use MKDoc::Util::Text2HTML;
use XML::Parser;
use Exporter;
use strict;
use 5.6.0;
use utf8;

our @ISA = qw /MKDoc::XML::Tree/;


##
# $self->linkify (@links);
# ------------------------
#   Linkifies this node with @links
#
#   @params  : @links - array of links, a link being a hashref
#              specifying href, expr and desc
#   @returns : void
##
sub linkify
{
    my $self = shift;
    my @links = @_;
    my @nodes = $self->all_nodes;
    
    while (@nodes)
    {
	my $node = shift (@nodes);
	
	next unless (defined $node->{is_text} and $node->{is_text});
	next if ($node->{text} =~ /^(\s|\n|\r)*$/);
	next unless ($node->_is_linkifiable);
	
	foreach my $link (@links)
	{
	    my @split = _split_3 ($node->{text}, $link->{expr});
	    if (@split)
	    {
		if ($split[0] eq '' and $split[2] eq '')
		{
		    my $link_node   = $self->new ( 'a', [ { href => $link->{href}, title => $link->{desc} }, '0', $split[1] ] );
		    $node->replace_with ($link_node);
		}
		elsif ($split[0] eq '')
		{
		    my $link_node   = $self->new ( 'a', [ { href => $link->{href}, title => $link->{desc} }, '0', $split[1] ] );
		    my $text_after  = $self->new ( '0', $split[2] );
		    $node->replace_with ($link_node, $text_after);
		    unshift (@nodes, $text_after);
		}
		elsif ($split[2] eq '')
		{
		    my $text_before = $self->new ( '0', $split[0] );
		    my $link_node   = $self->new ( 'a', [ { href => $link->{href}, title => $link->{desc} }, '0', $split[1] ] );
		    $node->replace_with ($text_before, $link_node);
		    unshift (@nodes, $text_before);
		}
		else
		{
		    my $text_before = $self->new ( '0', $split[0] );
		    my $link_node   = $self->new ( 'a', [ { href => $link->{href}, title => $link->{desc} }, '0', $split[1] ] );
		    my $text_after  = $self->new ( '0', $split[2] );
		    $node->replace_with ($text_before, $link_node, $text_after);
		    unshift (@nodes, $text_before);
		    unshift (@nodes, $text_after);
		}
		last;
	    }
	}
    }
}


##
# InKana();
# ---------
#   @returns : returns the kana range for use 
#              in regex \p{InKana}
##
sub InKana
{
    return "3040 309F\n30A0 30FF";
}


##
# _split_3 ($text, $expr);
# ------------------------
#   @returns : A sub that splits a string in 3
#              bits when it needs hyperlinking
##
sub _split_3
{
    my $data  = shift;
    my $expr = shift;
    
    my $dlm = '(?:\p{IsSpace}|\p{IsPunct}|\&|\p{InKana})';
    my $re = join "(?:\\s|\\r|\\n)+", map { quotemeta ($_) } split /(?:\s|\r|\n)+/, $expr;

    return unless ($data =~ /$re/i);
    my ($token1, $token2, $token3);
    
    # the expression could be at the beginning...
    if ($data =~ /^$re$dlm/ism)
    {
	$token1 = '';
	($token2, $token3) = $data =~ /^($re)($dlm.*?)$/is;
    }
    
    # ... or in the middle ...
    elsif ($data =~ /$dlm$re$dlm/ism)
    {
	($token1, $token2, $token3) = $data =~ /^(.*?$dlm)($re)($dlm.*?)$/is;
    }
    
    # ... or at the end ...
    elsif ($data =~ /$dlm$re$/ism)
    {
	($token1, $token2) = $data =~ /(.*$dlm)($re)$/is;
	$token3 = '';
    }
    
    # ... or it could be an exact match ...
    elsif ($data =~ /^$re$/ism)
    {
	$token1 = '';
	($token2) = $data =~ /($re)/is;
	$token3 = '';
    }
    
    # ... or we don't match!
    else
    {
	return;
    }
    
    my @res = ($token1, $token2, $token3);
    u_convert_arrayref (\@res);
    return @res;
}


##
# $self->_linkifiable_nodes;
# --------------------------
#   This method finds out which text nodes are linkifiable
#   and returns them.
#
#   @returns : an array reference of linkifiable nodes
##
sub _linkifiable_nodes
{
    my $self = shift;
    my @all_nodes = $self->all_nodes;
    my @res = ();
    foreach my $node (@all_nodes)
    {
	push @res, $node
	    if ($node->{is_text} and $node->_is_linkifiable and $node !~ /^(\s|\n|\r)+$/);
    }
    return \@res;
}


##
# $self->_is_linkifiable;
# -----------------------
#   This method is meant to be called on text nodes only!
#   This is because element nodes are never linkifiable.
#
#   @returns : TRUE if $self is linkifiable, FALSE otherwise.
##
sub _is_linkifiable
{
    my $self = shift;
    my $parent = $self->{parent};
    
    return 1 unless (defined $parent);
    return if ( $self->{name} eq 'a'      or
		$self->{name} eq 'option' or
		$self->{name} eq 'select' );
    
    return $parent->_is_linkifiable;
}


##
# $self->_can_trim_spaces;
# ------------------------
#   Returns TRUE if $self ($self being a text node)
#   can be whitespace stripped
##
sub _can_trim_spaces
{
    my $self = shift;
    my $self_parent = $self->{parent};
    return 1 unless (defined $self_parent);
    
    return if ( $self->{name} eq 'code' or
		$self->{name} eq 'tt'   or
		$self->{name} eq 'samp' or
		$self->{name} eq 'kbd'  or
		$self->{name} eq 'pre'  or
		$self->{name} eq 'textarea' );
    
    return $self_parent->_can_trim_spaces;
}


##
# $self->trim_spaces;
# -------------------
#   Removes all unnecessary whitespaces,
#   carriage returns, etc
##
sub trim_spaces
{
    my $self = shift;
    if ($self->{is_text})
    {
	if ($self->_can_trim_spaces)
	{
	    my $text = $self->{text};
	    $text =~ s/(\s|\n|\r)+/ /gsm;
	    if ($text =~ /^\s*$/)
	    {
		my $previous = $self->left_sibling;
		my $next = $self->right_sibling;
		if ( defined $previous and $previous->is_inline and
		     defined $next and $next->is_inline )
		{
		    $self->{text} = ' ';
		}
		else
		{
		    $self->delete;
		}
	    }
	    else
	    {
		$self->{text} = $text;
	    }
	}
    }
    else
    {
	foreach my $child ($self->children)
	{
	    $child->trim_spaces;
	}
    }
}


##
# $self->is_inline;
# -----------------
#   Returns TRUE if the element is an inline element,
#   FALSE otherwise. This list should probably be
#   extracted from a config file at some point.
##
sub is_inline
{
    my $self = shift;
    
    # this is taken from this list
    # http://htmlhelp.com/reference/html40/olist.html
    my @list = qw /a applet basefont bdo br font iframe img
	           map area object param q script span sub sup
	           abbr acronym cite code del dfn em ins kbd
		   samp strong var
		   b big i s small strike tt u/;
    
    my $q_name = quotemeta ($self->{name});
    my $res = scalar grep (/^$q_name$/, @list);
    return $res;
}


##
# $self->text_nice;
# -----------------
#   Returns nicely indented XHTML, taking into account tags like
#   <pre>, <textarea> and <a>
##
sub text_nice
{
    my $self = shift;
    my $self_parent = $self->{parent};
    my $left_sibling = $self->left_sibling;
    my $indent = shift || 0;
    
    # if this is a piece of text that has an pre, textarea
    # or inline element as a parent then we have to return it
    # untouched
    if ( defined $self_parent and
	 ( $self_parent->is_inline or
	   $self_parent->{name} eq 'pre' or
	   $self_parent->{name} eq 'textarea') and
	 ( defined $self->{is_text} and
	   $self->{is_text} ) )
    {
	return text2xml ($self->{text});
    }
    
    # if this is a piece of text an inline element as a
    # left sibling, then we return it untouched
    elsif ( $self->{is_text} and
	    defined $left_sibling and
	    $left_sibling->is_inline )
    {
	return text2xml ($self->{text});
    }

    # we want inline elements to be inlined!
    elsif ( $self->is_inline )
    {
	my $name = $self->{name};
	my $attr = $self->_attributes_text;
	my @children = $self->children;
	unless (@children)
	{
	    if ( $name eq 'br'  or $name eq 'img'   or
		 $name eq 'hr'  or $name eq 'input' or
		 $name eq 'area' )
	    {
		if ($attr eq '') { return "<$name />"       }
		else             { return "<$name $attr />" }
	    }
	    else
	    {
		if ($attr eq '') { return "<$name></$name>"       }
		else             { return "<$name $attr></$name>" }
	    }
	}
	
	my @res = ();
	if ($attr eq '') { push @res, "<$name>"       }
	else             { push @res, "<$name $attr>" }
	
	foreach my $child (@children) { push @res, $child->text_nice ($indent + 1) }
	push @res, "</$name>";
	return join '', @res;
    }
    
    # pre or textarea have to be handled differently as well
    elsif ( $self->{name} eq 'pre' or
	    $self->{name} eq 'textarea' )
    {
	my $name = $self->{name};
	my $attr = $self->_attributes_text;
	my @children = $self->children;
	unless (@children)
	{
	    if ($attr eq '') { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name></$name>"       }
	    else             { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr></$name>" }
	}
	
	my @res = ();
	if ($attr eq '') { push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name>"       }
	else             { push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr>" }
	
	foreach my $child (@children) { push @res, $child->text_nice ($indent + 1) }
	push @res, "</$name>";
	return join '', @res;
    }
    
    # otherwise treat everything as XML
    else
    {	
	if ($self->{is_text})
	{
	    return "\n" . MKDoc::XML::Tree::INDENT x $indent . text2xml ($self->{text});
	}
	
	my $name = $self->{name};
	my $attr = $self->_attributes_text;
	my @children = $self->children;
	unless (@children)
	{
	    if ( $name eq 'br'  or $name eq 'img'   or
		 $name eq 'hr'  or $name eq 'input' or
		 $name eq 'area' )
	    {
		if ($attr eq '') { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name />"       }
		else             { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr />" }
	    }
	    else
	    {
		if ($attr eq '') { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name></$name>"       }
		else             { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr></$name>" }
	    }
	}
	
	my @res = ();
	if ($attr eq '') { push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name>"       }
	else             { push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr>" }
	
	foreach my $child (@children) { push @res, $child->text_nice ($indent + 1) }
	push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "</$name>";
	return join '', @res;
    }
}


1;







