# -------------------------------------------------------------------------------------
# flo::editor::File
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      A class that can be used to attach any kind of file to an MKDoc document
#
# -------------------------------------------------------------------------------------
package flo::editor::File;
use MKDoc::Unicode;
use MKDoc::Util::Text2HTML;
use flo::Standard;
use flo::Editor;
use strict;
use 5.6.1;
use utf8;

use flo::editor::Mixin::compute_name;
use flo::editor::Mixin::normalize_name;
use flo::editor::Mixin::upload_file;
our @ISA = qw /
  flo::editor::Mixin::compute_name
  flo::editor::Mixin::normalize_name
  flo::editor::Mixin::upload_file
    /;


##
# $self->new;
# -----------
#   Returns a new flo::editor::File object based on CGI input
##
sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};
    
    # if CGI is defined, then we probably want to do some stuff
    if (defined $cgi)
    {
	# update title field
	if (defined $cgi->param ($param_name . "_title"))
	{
	    my $title = $cgi->param ($param_name . "_title");
	    u_mark (\$title);
	    $self->{title} = $title;
	}
	else
	{
	    my $empty = "";
	    u_mark (\$empty);
	    $self->{title} = $empty;
	}
	    
	# if the upload field is not empty, then the user
	# just uploaded a new file
	if (defined $cgi->param ($param_name . "_upload"))
	{
	    my $new_file = $cgi->param ($param_name);
	    if ($new_file)
	    {
		my $new_file_friendly = $self->normalize_name ($new_file);
		my $dir = $self->compute_name;
		my $full_dir = general_config->get ('file_disk_path') . "/" . $dir;
		
		mkdir $full_dir;
		$class->upload_file ('file_disk_path', $dir, $new_file_friendly, $cgi->param ($param_name));
		
		my $self_title = $self->{title} || $new_file_friendly;
		u_mark (\$self_title);
		
		my $self_file = "$dir/$new_file_friendly";
		u_mark (\$self_file);
		
		$self->{file} = $self_file;
		$self->{title} = $self_title;
	    }
	}
	else
	{
	    my $file = $cgi->param ($param_name . "_file");
	    u_mark (\$file);
	    $self->{file} = $file;
	}
    }
    
    $self->{class} = 'file';
    return $self;
}


##
# $self->template_path;
# ---------------------
#   Computes the path of the template which has to be used for
#   that component given the current document language
##
sub template_path
{
    my $self = shift;
    
    my $document_table = table ('Document');
    my $document = $document_table->get ( Full_Path => path_info );
    my $lang = $document->{Lang};
    
    return template ('editor/file', $lang);
}


##
# $self->edit;
# ------------
#   Return the edit HTML chunk for that component
##
sub edit
{
    my $self = shift;
    my $cgi  = $self->{cgi};
    return new flo::Template ( file => $self->template_path,
			       hash => {
				   block_name  => $self->{param_name},
				   file        => text2html ($self->{file}),
				   title       => text2html ($self->{title}),
				   file_url    => general_config->get ("file_http_path") . "/" . $self->{file},
				   Direction   => current_document->Direction,
				   Align       => current_document->Align,
			       } )->parse;
}


##
# $self->generate_xml;
# --------------------
#   Returns the XML for that component
##
sub generate_xml
{
    my $self = shift;
    my $data = '';
    if ($self->{file})
    {
	my $src = general_config->get ("file_http_path") . "/" . $self->{file};
	my $title = text2html ($self->{title});
	$data = qq |<a href="$src">$title</a> />|;
    }
    
    return $data;
}


##
# $self->parse_xml;
# -----------------
#   Parses the XML for that component
##
sub parse_xml
{
    my $self = shift;
    my $line = u_shift (\@_);
    if ($line)
    {
	my ($file, $title) = $line =~ /\<a href=\"(.*?)\"\>(.*)\<\/a\> \/\>/;
	u_mark (\$file);
	u_mark (\$title);
	my $q_file_http_path = quotemeta (general_config->get ("file_http_path"));
	$file =~ s/^$q_file_http_path//o;
	$file =~ s/^\///o;
	$self->{file}  = $file;
	$self->{title} = $title || $self->{file};
    }
    else
    {
	my $empty = "";
	u_mark (\$empty);
	$self->{file} = $empty;
	$self->{title} = $empty;
    }
}


##
# $self->html;
# ------------
#   Generates the (X)HTML for that component
##
sub html
{
    my $self = shift;
    return $self->generate_xml;
}


sub Align { return current_document->Align }
sub Direction { return current_document->Direction }


1;
